package videoconference.ui;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.os.IBinder;

import androidx.core.app.NotificationManagerCompat;

import com.evolgence.myspear.R;
import com.flashphoner.fpwcsapi.Flashphoner;
import com.flashphoner.fpwcsapi.bean.Connection;
import com.flashphoner.fpwcsapi.bean.Data;
import com.flashphoner.fpwcsapi.constraints.AudioConstraints;
import com.flashphoner.fpwcsapi.session.Call;
import com.flashphoner.fpwcsapi.session.CallOptions;
import com.flashphoner.fpwcsapi.session.Session;
import com.flashphoner.fpwcsapi.session.SessionEvent;
import com.flashphoner.fpwcsapi.session.SessionOptions;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.product.download.Utils.LogUtil;

import org.webrtc.MediaConstraints;

import java.util.Map;

public class SipService extends Service implements SessionEvent {


    public static final String ACTION_SIP_REGISTRATION_CHANGE = "SPEARON.VC.SIP.REGISTRATION_CHANGE";
    public static final String ACTION_SIP_REGISTRATION = "SPEARON.VC.SIP.REGISTRATION";
    private static final String PRT_CHANNEL_SILENT_ID = "sip_channel_id";
    private static final CharSequence PRT_CHANNEL_SILENT_NAME = "sip channel";

    private final int SERVICE_NOTIFICATION = 66666;





    private Session session;


    private NotificationManagerCompat notificationManager;

    private String phoneNumber;

    private Call call;


    @Override
    public void onCreate() {
        super.onCreate();


        notificationManager = NotificationManagerCompat.from(getApplicationContext());


    }

    private void showServiceNotifiCation() {


        Intent intentSwitch = new Intent(getApplicationContext(), TestSipPhoneActivity.class);
        intentSwitch.setFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);

        PendingIntent pendingIntentSwitch =
                PendingIntent.getActivity(this, 34, intentSwitch, PendingIntent.FLAG_UPDATE_CURRENT);

        Notification.Builder builder;

        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            NotificationChannel serviceChannel = new NotificationChannel(
                    PRT_CHANNEL_SILENT_ID, PRT_CHANNEL_SILENT_NAME,
                    NotificationManager.IMPORTANCE_LOW
            );

            serviceChannel.setLockscreenVisibility(Notification.VISIBILITY_PUBLIC);
            serviceChannel.setShowBadge(true);

            NotificationManager manager = getSystemService(NotificationManager.class);
            manager.createNotificationChannel(serviceChannel);

            builder = new Notification.Builder(this, PRT_CHANNEL_SILENT_ID);

        } else {
            builder = new Notification.Builder(this);
        }


        builder.setSmallIcon(R.drawable.ic_launcher)
                .setContentTitle(getString(R.string.app_name))
                .setContentText("Sip Call is running")
                .setPriority(Notification.PRIORITY_MAX)
                .setContentIntent(pendingIntentSwitch)
                .build();

        startForeground(SERVICE_NOTIFICATION, builder.build());


    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        int result = super.onStartCommand(intent, flags, startId);

        if (intent != null) {
            try {

                phoneNumber = intent.getStringExtra("bn");
                showServiceNotifiCation();

                if (ACTION_SIP_REGISTRATION.equals(intent.getAction())) {

                    registerToServer();

                }

            } catch (Exception e) {
               e.printStackTrace();
            }


        }


        return result;
    }


    @Override
    public void onDestroy() {


        try {


            try {
                if (call != null) {
                    call.hangup();
                }
                if (session != null) {
                    session.disconnect();
                }
            } catch (Exception e) {
               e.printStackTrace();
            }

            NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
            notificationManager.cancelAll();

            super.onDestroy();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    public void registerToServer() {


        try {

                    SessionOptions sessionOptions = new SessionOptions("wss://101.53.154.203:8443");
                    session = Flashphoner.createSession(sessionOptions);

                    Connection connection = new Connection();
                    connection.setSipLogin("test");
                    connection.setSipPassword("password");
                    connection.setSipDomain("sip domain");
                    connection.setSipOutboundProxy("");

                    connection.setSipPort(5060);
                    connection.setSipRegisterRequired(true);
                    session.on(this);
                    session.connect(connection);

        } catch (Exception e) {
           e.printStackTrace();
        }

    }








    @Override
    public void onAppData(Data data) {

    }

    @Override
    public void onConnected(Connection connection) {

        // showTipMessage("onConnected", connection.getStatus());
    }

    @Override
    public void onRegistered(Connection connection) {

        myCallStarted();

    }

    @Override
    public void onDisconnection(Connection connection) {

    }


    private void myCallStarted() {
        try {

            /**
             * Get call options from the callee text field
             */


            CallOptions callOptions = new CallOptions(phoneNumber);

            AudioConstraints audioConstraints = callOptions.getConstraints().getAudioConstraints();
            MediaConstraints mediaConstraints = audioConstraints.getMediaConstraints();

            mediaConstraints.optional.add(
                    new MediaConstraints.KeyValuePair("googEchoCancellation", Boolean.toString(true)));
            mediaConstraints.optional.add(
                    new MediaConstraints.KeyValuePair("googAutoGainControl", Boolean.toString(true)));
            mediaConstraints.optional.add(
                    new MediaConstraints.KeyValuePair("googNoiseSupression", Boolean.toString(true)));
            mediaConstraints.optional.add(
                    new MediaConstraints.KeyValuePair("googHighpassFilter", Boolean.toString(false)));
            mediaConstraints.optional.add(
                    new MediaConstraints.KeyValuePair("googEchoCancellation2", Boolean.toString(true)));
            mediaConstraints.optional.add(
                    new MediaConstraints.KeyValuePair("googAutoGainControl2", Boolean.toString(true)));
            mediaConstraints.optional.add(
                    new MediaConstraints.KeyValuePair("googNoiseSuppression2", Boolean.toString(true)));


            try {
                Map<String, String> inviteParameters = new Gson().fromJson("{header:value}",
                        new TypeToken<Map<String, String>>() {
                        }.getType());
                callOptions.setInviteParameters(inviteParameters);
            } catch (Exception e) {
                LogUtil.vLog("Invite Parameters have wrong format of json object");
            }
            call = session.createCall(callOptions);


            call.call();

            Flashphoner.getAudioManager().setUseSpeakerPhone(true);

        } catch (Exception e) {
           e.printStackTrace();
        }
    }


}


